@@ -9,6 +9,7 @@ Ace::Browser::AceSubs - Subroutines for AceBrowser
   use Ace;
   use Ace::Browser::AceSubs;
   use CGI qw(:standard);
+  use CGI::Cookie;
 
   my $obj = GetAceObject() || AceNotFound();
   PrintTop($obj);
@@ -38,6 +39,7 @@ The following subroutines are exported by default:
 
 The following subroutines are exported if explicitly requested:
 
+  AceAddCookie
   AceInit
   AceHeader
   AceMultipleChoices
@@ -102,8 +104,11 @@ use strict;
 use Ace::Browser::SiteDefs;
 use Ace 1.76;
 use CGI qw(:standard escape);
+use CGI::Cookie;
 
-use vars qw/@ISA @EXPORT @EXPORT_OK $VERSION %EXPORT_TAGS %DB %OPEN $HEADER $TOP/;
+use vars qw/@ISA @EXPORT @EXPORT_OK $VERSION %EXPORT_TAGS 
+  %DB %OPEN $HEADER $TOP @COOKIES
+  $APACHE_CONF/;
 
 require Exporter;
 @ISA = qw(Exporter);
@@ -114,8 +119,9 @@ $VERSION = 1.15;
 	     GetAceObject AceError AceNotFound AceMissing DoRedirect
 	     OpenDatabase Object2URL Url
 	     ObjectLink Configuration PrintTop PrintBottom);
-@EXPORT_OK = qw(AceRedirect Toggle ResolveUrl AceInit AceHeader TypeSelector Style
-	       Header Footer DB_Name AceMultipleChoices);
+@EXPORT_OK = qw(AceRedirect Toggle ResolveUrl AceInit AceAddCookie
+		AceHeader TypeSelector Style
+		Header Footer DB_Name AceMultipleChoices);
 %EXPORT_TAGS = ( );
 
 use constant DEFAULT_DATABASE  => 'default';
@@ -146,13 +152,25 @@ used for maintaining AceBrowser state.  It is not exported by default.
 
 =cut
 
+=item AceAddCookie(@cookies)
+
+This subroutine, which must be called b<after> OpenDatabase() and/or
+GetAceObject() and b<before> PrintTop(), will add one or more cookies
+to the outgoing HTTP headers that are emitted by AceHeader().  
+Cookies must be CGI::Cookie objects.
+
+=cut
+
+sub AceAddCookie {
+   push @COOKIES,@_;  # add caller's to our globals
+}
+
 ################## canned header ############
 sub AceHeader {
 
   my %searches = map {$_=>1} Configuration()->searches;
   my $quovadis = url(-relative=>1);
 
-  my @cookies;
   my $db = get_symbolic();
 
   my $referer  = referer();
@@ -164,7 +182,7 @@ sub AceHeader {
 			  -name=>"HOME_${db}",
 			  -value=>$referer,
 			  -path=>'/');
-    push(@cookies,$bookmark);
+    push(@COOKIES,$bookmark);
   }
 
   if ($searches{$quovadis}) {
@@ -177,13 +195,12 @@ sub AceHeader {
     my $last_search = cookie(-name=>"ACEDB_$db",
 			     -value=>$quovadis,
 			     -path=>'/');
-    push(@cookies,$search_data,$last_search);
+    push(@COOKIES,$search_data,$last_search);
   }
 
+  print @COOKIES ? header(-cookie=>\@COOKIES,@_) : header(@_);
 
-  print header(-cookie=>\@cookies,@_) if @cookies;
-  print header(@_)               unless @cookies;
-
+  @COOKIES = ();
   $HEADER++;
 }
 
@@ -201,8 +218,9 @@ called by PrintTop() and Header() internally.
 # undefined database name.  Check the return code from this function and
 # return immediately if not true (actually, not needed because we exit).
 sub AceInit   {
-  $HEADER = 0;
-  $TOP    = 0;
+  $HEADER   = 0;
+  $TOP      = 0;
+  @COOKIES  = ();
 
   # keeps track of what sections should be open
   %OPEN = param('open') ? map {$_ => 1} split(' ',param('open')) : () ;
@@ -724,10 +742,14 @@ sub ResolveUrl {
     my ($url,$param) = @_;
     my ($main,$query,$frag) = $url =~ /^([^?\#]+)\??([^\#]*)\#?(.*)$/ if defined $url;
     $main ||= '';
+    
+    if (!defined $APACHE_CONF) {
+      $APACHE_CONF = eval { Apache->request->dir_config('AceBrowserConf') } ? 1 : 0;
+    }
 
     $main = Configuration()->resolvePath($main) unless $main =~ m!^/!;
     if (my $id = get_symbolic()) {
-      $main .= "/$id" unless $main =~ /$id/;
+      $main .= "/$id" unless $main =~ /$id/ or $APACHE_CONF;
     }
 
     $main .= "?$query" if $query; # put the query string back
